"""Defects of pipeline."""


class Type:
    """Type of defect."""

    MetalLoss = "metalloss"
    Dent = "dent"
    Crack = "crack"


class Base(object):
    """Base class for different defect types."""

    def __init__(self, defect_type, pipe):
        """Create new defect at the pipe."""
        self.type = defect_type
        self.pipe = pipe

    def __str__(self):
        """Return as text."""
        return self.type



"""Single pipe from the pipeline."""
from . import Error as ErrorBase

CIRCLE_MINUTES = 60.0


class ErrDefectSize(ErrorBase):
    """Wrong defect size (outside pipe)."""


class ErrDefectDepth(ErrorBase):
    """Wrong defect depth."""


class ErrDefectOrientStart(ErrorBase):
    """Wrong defect start orientdtion."""


class ErrDefectOrientLength(ErrorBase):
    """Wrong defect orientdtion length."""


class Pipe:
    """Single pipe under pressure (maop)."""

    def __init__(self, length, diameter, wallthickness, material, maop):
        """Create new pipe."""
        self.length = length  # mm
        self.diameter = diameter  # mm
        self.wallthickness = wallthickness  # mm
        self.material = material
        self.maop = maop
        self.metal_loss = []

    def __str__(self):
        """Return as text."""
        return "{} diam {} wall {} maop {}".format(
          str(self.material),
          self.diameter,
          self.wallthickness,
          self.maop
        )

    def add_metal_loss(self, start, length, orient_start, orient_length, depth):
        """Add and return metal loss defect at the pipe."""
        from .defect.metal_loss import Item

        if (start + length) > self.length:
            raise ErrDefectSize(
              "Defect border outside pipe: start {} + length {} = {} > pipe length {}.".format(
                start, length, start + length, self.length
              )
            )

        if depth > self.wallthickness:
            raise ErrDefectDepth("Defect depth {} > pipe wall thickness {}.".format(depth, self.wallthickness))

        if not (-CIRCLE_MINUTES <= orient_start <= CIRCLE_MINUTES):
            raise ErrDefectOrientStart(
              "Defect start orientdtion {} must be from {} to {} angle minutes.".format(
                orient_start, -CIRCLE_MINUTES, CIRCLE_MINUTES
              )
            )

        if not (0 < orient_length <= CIRCLE_MINUTES):
            raise ErrDefectOrientLength(
              "Defect orientdtion length {} must be from 0 to {} angle minutes.".format(
                orient_length, CIRCLE_MINUTES
              )
            )

        self.metal_loss.append(Item(self, start, length, orient_start, orient_length, depth))
        return self.metal_loss[-1]



"""Localization stuff."""
from py23 import open_text_file, gen_next


class Lang:
    """Embedded languages for localize explain text."""

    Ru = 'ru'


def fake_gettext(text, context):
    """Translate text in given context."""
    if isinstance(context.is_explain, dict):
        return context.is_explain.get(text, text)

    return text


def read_item(sign, line, inp):
    """Read multiline text item from given start line and input.

    Return tuple from item and last read line.
    """
    item = [line.strip().split(sign)[1].strip('"')]
    line = gen_next(inp).strip()
    while line.startswith('"'):
        item.append(line.strip('"'))
        line = gen_next(inp).strip()

    return (''.join(item), line)


def read_pair(line, inp):
    """Read and return pair msgid, msgstr from given start line and input."""
    msgid, line = read_item('msgid ', line, inp)
    msgstr, line = read_item('msgstr ', line, inp)

    return (msgid, msgstr)


def load_po(file_name):
    """Load data from given po file and return it as dict."""
    data = {}
    with open_text_file(file_name, 'r', 'utf-8') as inp:
        for line in inp:
            if line.startswith('msgid '):
                msgid, msgstr = read_pair(line, inp)
                if msgid and msgstr:
                    data[msgid] = msgstr

    return data



"""Metal loss defect."""
from . import Type, Base


class Item(Base):
    """Metal loss defect class."""

    def __init__(
      self, pipe, start, length, orient_start, orient_length, depth,
      max_depth_start=None, max_depth_orient=None
    ):
        """Create new defect."""
        super(Item, self).__init__(Type.MetalLoss, pipe)
        self.start = start  # mm
        self.length = length  # mm
        self.orient_start = orient_start  # minutes
        self.orient_length = orient_length  # minutes
        self.depth = depth  # mm
        self.max_depth_start = max_depth_start
        self.max_depth_orient = max_depth_orient

    def __str__(self):
        """Return as text."""
        return "{} len {} depth {}".format(
          super(Item, self).__str__(),
          self.length,
          self.depth
        )



"""ASME B31G method for metal loss defects edition 2012.

https://edu.truboprovod.ru/kbase/doc/start/WebHelp_ru/ASMEB31G.htm
"""
import math

from ...i18n import fake_gettext as _
from ... import Error as ErrorBase
from . import Context as ContextBase, EXPL_ROUND


class ErrMaterialSMTSNotDefined(ErrorBase):
    """SMTS not defined for material of the pipe."""


class Context(ContextBase):
    """Context of the ASME B31G method edition 2012."""

    name = "ASME B31G 2012"
    REPAIR_NOT_REQUIRED = 777

    def __init__(self, defect):
        """Check for defined material SMTS."""
        if defect.pipe.material.smts is None:
            raise ErrMaterialSMTSNotDefined("SMTS not defined for material of the pipe.")

        super(Context, self).__init__(defect)

    @property
    def relative_depth(self):
        """Return relative depth to wall thickness."""
        return float(self.anomaly.depth) / self.anomaly.pipe.wallthickness

    @property
    def z_param(self):
        """Parameter z."""
        pipe = self.anomaly.pipe
        z_val = pow(self.anomaly.length, 2) / (pipe.diameter * pipe.wallthickness)

        self.add_explain([
          '\n', _("Parameter Z = length^2 / (diameter * wallthickness).", self),
          '\n', "Z = {}^2 / ({} * {}) = {}.".format(
            round(self.anomaly.length, EXPL_ROUND),
            round(pipe.diameter, EXPL_ROUND),
            round(pipe.wallthickness, EXPL_ROUND),
            round(z_val, EXPL_ROUND),
          ),
        ])

        return z_val

    def m_param(self, z_val):
        """Parameter M."""
        m_val = math.sqrt(1 + 0.8 * z_val)

        self.add_explain([
          '\n', _("Parameter M = sqrt(1 + 0.8 * Z).", self),
          '\n', "M = sqrt(1 + 0.8 * {}) = {}.".format(
            round(z_val, EXPL_ROUND),
            round(m_val, EXPL_ROUND),
          ),
        ])

        return m_val

    def s_flow(self):
        """Return S_flow."""
        material = self.anomaly.pipe.material
        s_f = 1.1 * material.smys

        self.add_explain([
          '\n', _("Parameter Sflow = 1.1 * material_smys.", self),
          '\n', "Sflow = 1.1 * {} = {}.".format(
            round(material.smys, EXPL_ROUND),
            round(s_f, EXPL_ROUND),
          ),
        ])

        if s_f > material.smts:

            self.add_explain([
              '\n', _("Parameter Sflow > material SMTS.", self),
              '\n', _("Use material SMTS as Sflow = {}.", self).format(
                round(material.smts, EXPL_ROUND),
              ),
            ])

            return material.smts

        return s_f

    def get_stress_fail(self):
        """Return estimated failure stress level."""
        self.add_explain([
          '\n', _("Calculate failure stress level by the classic way.", self),
        ])

        s_f = self.s_flow()
        d_t = self.relative_depth
        z_val = self.z_param

        if z_val <= 20:
            v23 = 2.0 / 3.0
            m_val = self.m_param(z_val)
            s_p = s_f * (1 - v23 * d_t) / (1 - v23 * d_t / m_val)

            self.add_explain([
              '\n', _("Parameter Z = {} <= 20.", self).format(round(z_val, EXPL_ROUND)),
              '\n', _(
                "Failure stress level = Sflow * "
                "(1 - 2/3 * (depth / wallthickness)) / "
                "(1 - 2/3 * (depth / wallthickness) / M).",
                self
              ),

              '\n', self.explain_stress_fail(s_f, m_val, s_p),
            ])

            return s_p

        s_p = s_f * (1 - d_t)

        self.add_explain([
          '\n', _("Parameter Z = {} > 20.", self).format(round(z_val, EXPL_ROUND)),
          '\n', _("Failure stress level = Sflow * (1 - depth / wallthickness).", self),
          '\n', "stress_fail = {} * (1 - {} / {}) = {}.".format(
            round(s_f, EXPL_ROUND),
            round(self.anomaly.depth, EXPL_ROUND),
            round(self.anomaly.pipe.wallthickness, EXPL_ROUND),
            round(s_p, EXPL_ROUND),
          ),
        ])

        return s_p

    def explain_stress_fail(self, s_f, m_val, s_p):
        """Return text that explain stress_fail calculation."""
        return "stress_fail = {} * (1 - 0.85 * ({} / {})) / (1 - 0.85 * ({} / {}) / {}) = {}.".format(
          round(s_f, EXPL_ROUND),
          round(self.anomaly.depth, EXPL_ROUND),
          round(self.anomaly.pipe.wallthickness, EXPL_ROUND),
          round(self.anomaly.depth, EXPL_ROUND),
          round(self.anomaly.pipe.wallthickness, EXPL_ROUND),
          round(m_val, EXPL_ROUND),
          round(s_p, EXPL_ROUND),
        )

    def get_stress_fail_mod(self):
        """Return estimated failure stress level by modified method."""
        self.add_explain([
          '\n', _("Calculate failure stress level by the modified way.", self),
        ])

        s_f = self.s_flow()
        d_t = self.relative_depth
        z_val = self.z_param

        if z_val <= 50:
            m_val = math.sqrt(1 + 0.6275 * z_val - 0.003375 * pow(z_val, 2))
            self.add_explain([
              '\n', _("Parameter Z = {} <= 50.", self).format(round(z_val, EXPL_ROUND)),
              '\n', _("Parameter M = sqrt(1 + 0.6275 * Z - 0.003375 * Z^2)", self),
              '\n', "M = sqrt(1 + 0.6275 * {} - 0.003375 * {}^2) = {}".format(
                round(z_val, EXPL_ROUND),
                round(z_val, EXPL_ROUND),
                round(m_val, EXPL_ROUND),
              ),
            ])
        else:
            m_val = 0.032 * z_val + 3.3
            self.add_explain([
              '\n', _("Parameter Z = {} > 50.", self).format(round(z_val, EXPL_ROUND)),
              '\n', _("Parameter M = 0.032 * Z + 3.3", self),
              '\n', "M = 0.032 * {} + 3.3 = {}".format(
                round(z_val, EXPL_ROUND),
                round(m_val, EXPL_ROUND),
              ),
            ])

        s_p = s_f * (1 - 0.85 * d_t) / (1 - 0.85 * d_t / m_val)

        self.add_explain([
          '\n', _(
            "Failure stress level = Sflow * "
            "(1 - 0.85 * (depth / wallthickness)) / "
            "(1 - 0.85 * (depth / wallthickness) / M).",
            self
          ),
          '\n', self.explain_stress_fail(s_f, m_val, s_p),
        ])

        return s_p

    def get_press_fail(self, is_mod=False):
        """Return estimated failure pressure."""
        if is_mod:
            s_f = self.get_stress_fail_mod()
        else:
            s_f = self.get_stress_fail()

        pipe = self.anomaly.pipe
        p_f = 2 * s_f * pipe.wallthickness / pipe.diameter

        self.add_explain([
          '\n', _("Failure pressure = 2 * stress_fail * wallthickness / diameter.", self),
          '\n', "press_fail = 2 * {} * {} / {} = {}.".format(
            round(s_f, EXPL_ROUND),
            round(pipe.wallthickness, EXPL_ROUND),
            round(pipe.diameter, EXPL_ROUND),
            round(p_f, EXPL_ROUND),
          ),
        ])

        return p_f

    def erf(self, is_mod=False):
        """Return estimated repair factor."""
        modname = _("modified", self) if is_mod else _("classic", self)
        self.add_explain([
          _("Calculate ERF by {} {}.", self).format(self.name, modname),
        ])

        self.safe_pressure = self.get_press_fail(is_mod=is_mod)
        erf_val = self.anomaly.pipe.maop / self.safe_pressure

        self.add_explain([
          '\n', _("ERF = pipe_maop / press_fail.", self),
          '\n', "ERF = {} / {} = {}".format(
            self.anomaly.pipe.maop,
            round(self.safe_pressure, EXPL_ROUND),
            round(erf_val, EXPL_ROUND)
          ),
        ])

        return erf_val

    def years(self, is_mod=False):
        """Return estimated years for repair."""
        is_explain = self.is_explain

        erf_l = self.erf(is_mod=is_mod)
        if erf_l >= 1:
            self.add_explain([
              '\n', _("Repair required immediately, years to repair: 0.", self),
            ])
            return 0

        depth_saved = self.anomaly.depth

        right = int((self.anomaly.pipe.wallthickness - self.anomaly.depth) / self.corrosion_rate) + 1
        self.anomaly.depth = self.anomaly.pipe.wallthickness - self.corrosion_rate / 12.0

        self.add_explain([
          '\n',
          '\n', _("Repair is not required at the moment, calculate the time before repair.", self),
          '\n',
          _(
            "With corrosion rate {} mm/year, pipe wall {} and depth {} "
            "a through hole is formed after years: {}.",
            self
          ).format(
            round(self.corrosion_rate, EXPL_ROUND),
            self.anomaly.pipe.wallthickness,
            depth_saved,
            right
          ),
        ])

        self.is_explain = False
        erf_r = self.erf(is_mod=is_mod)
        self.is_explain = is_explain

        if erf_r < 1:
            self.add_explain([
              '\n', _("But even a through defect does not require repair.", self),
              '\n', _("ERF = {}. Use special value for years: 777.", self).format(
                round(erf_r, EXPL_ROUND)
              ),
            ])

            self.anomaly.depth = depth_saved
            return self.REPAIR_NOT_REQUIRED

        self.add_explain([
          '\n',
          _("Calculating the year in which the corrosion growth of the defect will require repair.", self),
        ])

        self.is_explain = False
        left = 0

        while (right - left) > 1:
            years = left + int((right - left) / 2)
            self.anomaly.depth = depth_saved + self.corrosion_rate * years
            erf_val = self.erf(is_mod=is_mod)
            if erf_val < 1:
                erf_l = erf_val
                left = years
            else:
                erf_r = erf_val
                right = years

        self.is_explain = is_explain
        self.add_explain([
          '\n', _("Years: {} ERF: {}.", self).format(left, round(erf_l, EXPL_ROUND)),
          '\n', _("Years: {} ERF: {}.", self).format(right, round(erf_r, EXPL_ROUND)),
          '\n', _("Defect will require repair after years: {}.", self).format(left),
        ])

        self.anomaly.depth = depth_saved
        return left



"""Danger calculation methods."""
from .. import Error as ErrorBase


class ErrDefectTypeNotSupported(ErrorBase):
    """Given defect type not supported by method."""


class Context(object):
    """Base class for Context of the danger method."""

    name = "not defined"
    valid_defect_types = []

    def __init__(self, defect):
        """Create new context instance."""
        if defect.type not in self.valid_defect_types:
            raise ErrDefectTypeNotSupported("Defect type '{}' not supported by method '{}'".format(
              defect.type, self.name
            ))

        self.anomaly = defect
        self.explain_text = []
        self.is_explain = False

    def __str__(self):
        """Return as text."""
        return "{}\nPipe {}\nDfct {}".format(
          self.name,
          str(self.anomaly.pipe),
          str(self.anomaly)
        )

    @classmethod
    def lang(cls, _lang_code):
        """Load language dict for localize explain text."""
        raise NotImplementedError("{}.lang".format(cls.__class__.__name__))

    def explain(self):
        """Return text with explanation for calculation."""
        return ''.join(self.explain_text)

    def add_explain(self, msg_list):
        """Add messages from the list to explain array."""
        if self.is_explain:
            self.explain_text.extend(msg_list)



"""ASME B31G method for metal loss defects.

https://pypi.org/project/pyintegrity/
https://github.com/novanumeric/WebIntegrity
"""
import os

from .. import Context as ContextBase
from ...defect import Type
from ...i18n import load_po

EXPL_ROUND = 3


class Context(ContextBase):
    """Context of the ASME B31G method."""

    valid_defect_types = [Type.MetalLoss]

    # ANSI/NACE SP0502-2010 (formerly RP0502) Appendix C: Postassessment: Corrosion Rate Estimation C3.2
    # https://github.com/vb64/pipeline.integrity/blob/main/docs/SOP_Pipeline_External_Corrosion.pdf
    corrosion_rate = 0.4  # mm/year

    def __init__(self, defect):
        """ASME B31G context."""
        super(Context, self).__init__(defect)
        self.safe_pressure = None

    @classmethod
    def lang(cls, lang_code):
        """Load language dict for localize explain text."""
        name = os.path.join(os.path.dirname(__file__), 'locale', lang_code, 'LC_MESSAGES', 'messages.po')
        return load_po(name)



"""ASME B31G method edition 1991."""
import math

from ...i18n import fake_gettext as _
from . import Context as ContextBase, EXPL_ROUND

DEPTH_OK_PERCENT = 10
DEPTH_CRITICAL_PERCENT = 80


class State:
    """State of pipe with defect."""

    Ok = 0
    Safe = 1
    Defected = 2
    Repair = 3
    Replace = 100


class Context(ContextBase):
    """Context of the ASME B31G  edition 1991."""

    name = "ASME B31G 1991"
    design_factor = 0.72  # DesignFactors.md
    temperature_factor = 1

    @property
    def relative_depth(self):
        """Return defect depth as percent from pipe wall thickness."""
        return 100.0 * self.anomaly.depth / self.anomaly.pipe.wallthickness

    @property
    def is_ok(self):
        """Return True if state is ok."""
        return self.relative_depth <= DEPTH_OK_PERCENT

    @property
    def is_replace(self):
        """Return True if state is replace."""
        return self.relative_depth >= DEPTH_CRITICAL_PERCENT

    def pipe_state(self, is_explain=False):
        """Return state for defected pipe."""
        self.is_explain = is_explain
        self.explain_text = []

        result = State.Repair

        self.add_explain([
          _("The relative defect depth == defect depth / pipe wall thickness * 100%.", self),
          '\n', "{} / {} * 100 = {}".format(
            self.anomaly.depth, self.anomaly.pipe.wallthickness, round(self.relative_depth, EXPL_ROUND)
          ),
        ])

        if self.is_ok:

            self.add_explain([
              '\n', _("The relative defect depth less than {}% from wall thickness.", self).format(
                DEPTH_OK_PERCENT
              ),
              '\n', _("The defect is not dangerous.", self),
            ])
            result = State.Ok

        elif self.is_replace:

            self.add_explain([
              '\n', _("The relative defect depth greater than {}% from wall thickness.", self).format(
                DEPTH_CRITICAL_PERCENT
              ),
              '\n', _("The defect needs to be repaired.", self),
            ])
            result = State.Replace

        else:

            max_length = self.defect_max_length()
            if max_length > self.anomaly.length:
                self.add_explain([
                  '\n',
                  _("The length of the defect {} does not exceed the maximum allowable length {}.", self).format(
                    self.anomaly.length, round(max_length, EXPL_ROUND)
                  ),
                  '\n', _("The defect is not dangerous.", self),
                ])

                result = State.Safe

            else:

                self.add_explain([
                  '\n',
                  _("The length of the defect {} exceed the maximum allowable length {}.", self).format(
                    self.anomaly.length, round(max_length, EXPL_ROUND)
                  ),
                  '\n', _("It is necessary to calculate the allowable pressure for defect.", self),
                ])

                self.safe_pressure = self.get_safe_pressure()
                if self.safe_pressure > self.anomaly.pipe.maop:
                    self.add_explain([
                      '\n',
                      _("The working pressure {} does not exceed the allowable pressure {}.", self).format(
                        self.anomaly.pipe.maop, round(self.safe_pressure, EXPL_ROUND)
                      ),
                      '\n', _("The defect is not dangerous.", self),
                    ])
                    result = State.Defected
                else:
                    self.add_explain([
                      '\n',
                      _("Repair or pressure reduction to {} required.", self).format(
                        round(self.safe_pressure, EXPL_ROUND)
                      ),
                    ])

        return result

    def get_b(self):
        """Parameter B from method description."""
        self.add_explain([
          '\n', _("Parameter B.", self),
        ])

        b_max = 4.0
        border = 17.5
        if self.relative_depth < border:
            self.add_explain([
              '\n',
              _("The relative defect depth {} less than {}%.", self).format(
                round(self.relative_depth, EXPL_ROUND), border
              ),
              '\n', _("Set Parameter B value to {}.", self).format(b_max),
            ])
            return b_max

        self.add_explain([
          '\n',
          _("The relative defect depth {} more than {}%.", self).format(
            round(self.relative_depth, EXPL_ROUND), border
          ),
        ])

        rel = self.relative_depth / 100.0
        b_val = math.sqrt(math.pow(rel / (1.1 * rel - 0.15), 2) - 1)

        self.add_explain([
          '\n',
          "B = sqrt(pow({} / (1.1 * {} - 0.15), 2) - 1) = {}".format(
            round(rel, EXPL_ROUND), round(rel, EXPL_ROUND), round(b_val, EXPL_ROUND)
          ),
        ])

        return b_val

    def get_a(self, max_length):
        """Parameter A from method description."""
        a_val = 0.823 * max_length / self.diam_wall

        pipe = self.anomaly.pipe
        self.add_explain([
          '\n', "A = 0.823 * defect_length / sqrt(diameter * wallthickness)",
          '\n', "A = 0.823 * {} / sqrt({} * {}) = {}".format(
            max_length, pipe.diameter, pipe.wallthickness, round(a_val, EXPL_ROUND)
          ),
        ])

        return a_val

    @property
    def diam_wall(self):
        """Intermediate parameter."""
        pipe = self.anomaly.pipe
        return math.sqrt(pipe.diameter * pipe.wallthickness)

    def defect_max_length(self):
        """Return maximum allowable longitudinal extent of corrosion."""
        self.add_explain([
          '\n',
          _("Calculation of the maximum allowable defect length.", self),
        ])
        b_val = self.get_b()
        length = 1.12 * b_val * self.diam_wall
        pipe = self.anomaly.pipe

        self.add_explain([
          '\n',
          "L = 1.12 * B * sqrt(diameter * wallthickness)",
          "L = 1.12 * {} * sqrt({} * {}) = {}".format(
            round(b_val, EXPL_ROUND), pipe.diameter, pipe.wallthickness, round(length, EXPL_ROUND)
          ),
        ])

        return length

    def get_design_pressure(self):
        """Return design pressure."""
        pipe = self.anomaly.pipe
        smys = pipe.material.smys
        p_v = 2.0 * smys * pipe.wallthickness * self.design_factor * self.temperature_factor / pipe.diameter

        self.add_explain([
          '\n',
          "Design_press = 2 * material_smys * wallthickness * design_factor * temperature_factor / diam.",
          '\n',
          "Design_press = 2 * {} * {} * {} * {} / {} = {}.".format(
            smys, pipe.wallthickness, self.design_factor, self.temperature_factor,
            pipe.diameter, round(p_v, EXPL_ROUND)
          ),
        ])

        return p_v

    def get_safe_pressure(self):
        """Return acceptable pressure level."""
        self.add_explain([
          '\n',
          _("Calculation of the maximum allowable pressure.", self),
          '\n', _("Parameter A for defect length {}.", self).format(self.anomaly.length),
        ])
        a_val = self.get_a(self.anomaly.length)

        self.add_explain([
          '\n', _("Design pressure.", self),
        ])
        p_val = self.get_design_pressure()

        d_t = self.relative_depth / 100.0
        tmp = 1.1 * p_val

        if a_val > 4.0:
            p_s = tmp * (1 - d_t)
            self.add_explain([
              '\n', _("Parameter A more than 4.", self),
              '\n', "Safe_press = 1.1 * design_press * (1 - rel_depth).",
              '\n', "Safe_press = 1.1 * {} * (1 - {}) = {}.".format(
                round(p_val, EXPL_ROUND), round(d_t, EXPL_ROUND), round(p_s, EXPL_ROUND)
              ),
            ])
        else:
            v23 = 2.0 / 3.0
            a_pow = math.sqrt(math.pow(a_val, 2) + 1)
            p_s = tmp * ((1 - v23 * d_t) / (1 - v23 * d_t / a_pow))
            self.add_explain([
              '\n', _("Parameter A less than 4.", self),
              '\n', "a_pow = sqrt(pow(a_param, 2) + 1).",
              '\n', "a_pow = sqrt(pow({}, 2) + 1) = {}.".format(
                round(a_val, EXPL_ROUND), round(a_pow, EXPL_ROUND)
              ),
              '\n',
              "Safe_press = 1.1 * design_press * ((1 - 2/3 * rel_depth) / (1 - 2/3 * rel_depth / a_pow)).",
              '\n', "Safe_press = 1.1 * {} * ((1 - 2/3 * {}) / (1 - 2/3 * {} / {})) = {}.".format(
                round(p_val, EXPL_ROUND), round(d_t, EXPL_ROUND), round(d_t, EXPL_ROUND),
                round(a_pow, EXPL_ROUND), round(p_s, EXPL_ROUND)
              ),
            ])

        if p_s > p_val:
            self.add_explain([
              '\n', _("Safe pressure {} more than design pressure {}.", self).format(
                round(p_s, EXPL_ROUND), round(p_val, EXPL_ROUND)
              ),
              '\n',
              _("Use design pressure {} as maximum allowable pressure.", self).format(round(p_val, EXPL_ROUND)),
            ])
            return p_val

        self.add_explain([
          '\n', _("Use safe pressure {} as maximum allowable pressure.", self).format(round(p_s, EXPL_ROUND)),
        ])
        return p_s
